/*
Bullet Continuous Collision Detection and Physics Library
Copyright (c) 2003-2006 Erwin Coumans  http://continuousphysics.com/Bullet/

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the use of this software.
Permission is granted to anyone to use this software for any purpose, 
including commercial applications, and to alter it and redistribute it freely, 
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/


///create 125 (5x5x5) dynamic object
#define ARRAY_SIZE_X 2
#define ARRAY_SIZE_Y 2
#define ARRAY_SIZE_Z 2

//maximum number of objects (and allow user to shoot additional boxes)
#define MAX_PROXIES (ARRAY_SIZE_X*ARRAY_SIZE_Y*ARRAY_SIZE_Z + 1024)

///scaling of the objects (0.1 = 20 centimeter boxes )
#define SCALING 1.
#define START_POS_X -5
#define START_POS_Y -5
#define START_POS_Z -3

#include "RollingFrictionDemo.h"
#include "GlutStuff.h"
///btBulletDynamicsCommon.h is the main Bullet include file, contains most common include files.
#include "btBulletDynamicsCommon.h"
#include "GLDebugDrawer.h"
static GLDebugDrawer sDebugDraw;
#include <stdio.h> //printf debugging


// Global physics parameters
#define PHYS_GLOBAL_SCALE_FACTOR (10.0f)

#define PHYS_SCALED(x) ((x) * PHYS_GLOBAL_SCALE_FACTOR)

// THIS IS HACKED - made to work with PHYS_GLOBAL_SCALE_FACTOR as a power of 10  - THIS IS HACKED//
#define PHYS_CONTACT_BREAK_THRESH (PHYS_SCALED(0.02f) * 0.1f)
// THIS IS HACKED - made to work with PHYS_GLOBAL_SCALE_FACTOR as a power of 10  - THIS IS HACKED//

// Time step
#define PHYS_FIXED_STEP_SIZE_S (1.0f / 120.0f)
#define PHYS_GRAVITY_Y_AXIS_SI PHYS_SCALED(-9.8f)

// Balls
#define PHYS_BALL_MASS (0.170f)
#define PHYS_BALL_RADIUS PHYS_SCALED(0.029f)
#define PHYS_BALL_FRICTION (0.2f)
#define PHYS_BALL_ROLL_FRICTION (0.1f)
#define PHYS_BALL_RESTITUTION (0.9f)
#define PHYS_BALL_LIN_DAMPING (0.0f)
#define PHYS_BALL_ANG_DAMPING (0.8f)
#define PHYS_BALL_LIN_SLEEP_THRESH (0.1f)
#define PHYS_BALL_ANG_SLEEP_THRESH (0.1f)

// Table floor
#define PHYS_FLOOR_FRICTION (0.3f)
#define PHYS_FLOOR_ROLL_FRICTION (0.1f)
#define PHYS_FLOOR_RESTITUTION (0.0f)

static bool pos_cam = true;
static btScalar impulse_trigger_time = 2.0f;
static bool impulse_should_trigger = true;

void RollingFrictionDemo::clientMoveAndDisplay()
{
    if(pos_cam){
        for(int i=0; i<14; i++){
            stepFront();
        }
        
        pos_cam = false;
    }
    
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

	//simple dynamics world doesn't handle fixed-time-stepping
	float ms = getDeltaTimeMicroseconds();
    float delta_seconds = ms / 1000000.f;
    impulse_trigger_time -= delta_seconds;
    
    if(impulse_trigger_time <= 0.0f && impulse_should_trigger){
        ball_rb->activate();
        ball_rb->applyCentralImpulse(btVector3(1.5f,0.0f, 0.5f));
        impulse_should_trigger = false;
    }
	
	///step the simulation
	if (m_dynamicsWorld)
	{
        m_dynamicsWorld->stepSimulation(delta_seconds, INT32_MAX, PHYS_FIXED_STEP_SIZE_S);
        
		//optional but useful: debug drawing
		m_dynamicsWorld->debugDrawWorld();
	}
		
	renderme(); 

	glFlush();

	swapBuffers();

}

void RollingFrictionDemo::displayCallback(void) {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
	
	renderme();

	//optional but useful: debug drawing to detect problems
	if (m_dynamicsWorld)
		m_dynamicsWorld->debugDrawWorld();

	glFlush();
	swapBuffers();
}

void RollingFrictionDemo::initPhysics()
{
	setTexturing(true);
	setShadows(true);

	setCameraDistance(btScalar(SCALING*10.));

    // WORLD
    {
        btDefaultCollisionConfiguration* _collisionConfiguration = new btDefaultCollisionConfiguration();
        btCollisionDispatcher* _dispatcher = new btCollisionDispatcher(_collisionConfiguration);
        btBroadphaseInterface* _broadphase = new btDbvtBroadphase();
        
        btSequentialImpulseConstraintSolver* sol = new btSequentialImpulseConstraintSolver;
        btConstraintSolver* _solver = sol;
        
        m_dynamicsWorld = new btDiscreteDynamicsWorld(_dispatcher, _broadphase, _solver, _collisionConfiguration);
        m_dynamicsWorld->setGravity(btVector3(0.0f ,PHYS_GRAVITY_Y_AXIS_SI, 0.0f));
        gContactBreakingThreshold = PHYS_CONTACT_BREAK_THRESH;
        
        // Tick callback for collision handling
        m_dynamicsWorld->setDebugDrawer(&sDebugDraw);
    }
    
    // FLOOR
    {
        btCollisionShape* collision_shape = new btStaticPlaneShape(btVector3(0.0f, 1.0f, 0.0f), 0.0f);
        
        btTransform T;
        T.setIdentity();
        T.setOrigin(btVector3(0.0, 0.0, 0.0));
        
        btVector3 local_inertia(0.0f, 0.0f, 0.0f);
        
        btDefaultMotionState* my_motion_state = new btDefaultMotionState(T);
        btRigidBody::btRigidBodyConstructionInfo rb_info(0.0f, my_motion_state, collision_shape, local_inertia);
        rb_info.m_additionalDamping = true;
        btRigidBody* ground_plane_rb = new btRigidBody(rb_info);
        
        ground_plane_rb->setFriction(PHYS_FLOOR_FRICTION);
        ground_plane_rb->setRestitution(PHYS_FLOOR_RESTITUTION);
        ground_plane_rb->setRollingFriction(PHYS_FLOOR_ROLL_FRICTION);
        
        m_dynamicsWorld->addRigidBody(ground_plane_rb);
    }

    // BALL
    {
        btTransform T;
        T.setIdentity();
        T.setOrigin(btVector3(0.0f, PHYS_BALL_RADIUS, 0.0f));
        
        
        btCollisionShape* collision_shape = new btSphereShape(PHYS_BALL_RADIUS);
        btVector3 local_inertia(0.0f, 0.0f, 0.0f);
        collision_shape->calculateLocalInertia(PHYS_BALL_MASS, local_inertia);
        
        btDefaultMotionState* my_motion_state = new btDefaultMotionState(T);
        btRigidBody::btRigidBodyConstructionInfo rb_info(PHYS_BALL_MASS, my_motion_state, collision_shape, local_inertia);
        rb_info.m_additionalDamping = true;
        ball_rb = new btRigidBody(rb_info);
        
        ball_rb->setFriction(PHYS_BALL_FRICTION);
        ball_rb->setRollingFriction(PHYS_BALL_ROLL_FRICTION);
        ball_rb->setRestitution(PHYS_BALL_RESTITUTION);
        ball_rb->setDamping(PHYS_BALL_LIN_DAMPING, PHYS_BALL_ANG_DAMPING);
        ball_rb->setSleepingThresholds(PHYS_BALL_LIN_SLEEP_THRESH, PHYS_BALL_ANG_SLEEP_THRESH);
        ball_rb->setCcdMotionThreshold(PHYS_BALL_RADIUS);
        ball_rb->setCcdSweptSphereRadius(PHYS_BALL_RADIUS);
        
        m_dynamicsWorld->addRigidBody(ball_rb);
        
    }
    
    // Barriers (Copy paste just for example)
    {
        // 1
        btVector3 hullPoints[4] = {btVector3(10.0,0.0,5.0),btVector3(10.0,0.0,-5.0)};
        btConvexHullShape hullB;
        hullB.addPoint(hullPoints[0]);
        hullB.addPoint(hullPoints[1]);
        
        btCollisionShape* collision_shape = new btConvexHullShape(hullB);
        
        btTransform T;
        T.setIdentity();
    
        btVector3 local_inertia(0.0f, 0.0f, 0.0f);
        btDefaultMotionState* my_motion_state = new btDefaultMotionState(T);
        btRigidBody::btRigidBodyConstructionInfo rb_info(0.0f, my_motion_state, collision_shape, local_inertia);
        rb_info.m_additionalDamping = true;
        btRigidBody* convex_hull_rb = new btRigidBody(rb_info);
        
        m_dynamicsWorld->addRigidBody(convex_hull_rb);
    }
    {
        // 2
        btVector3 hullPoints[4] = {btVector3(10.0,0.0,5.0),btVector3(-10.0,0.0,5.0)};
        btConvexHullShape hullB;
        hullB.addPoint(hullPoints[0]);
        hullB.addPoint(hullPoints[1]);
        
        btCollisionShape* collision_shape = new btConvexHullShape(hullB);
        
        btTransform T;
        T.setIdentity();
        
        btVector3 local_inertia(0.0f, 0.0f, 0.0f);
        btDefaultMotionState* my_motion_state = new btDefaultMotionState(T);
        btRigidBody::btRigidBodyConstructionInfo rb_info(0.0f, my_motion_state, collision_shape, local_inertia);
        rb_info.m_additionalDamping = true;
        btRigidBody* convex_hull_rb = new btRigidBody(rb_info);
        
        m_dynamicsWorld->addRigidBody(convex_hull_rb);
    }
    {
        // 3
        btVector3 hullPoints[4] = {btVector3(-10.0,0.0,5.0),btVector3(-10.0,0.0,-5.0)};
        btConvexHullShape hullB;
        hullB.addPoint(hullPoints[0]);
        hullB.addPoint(hullPoints[1]);
        
        btCollisionShape* collision_shape = new btConvexHullShape(hullB);
        
        btTransform T;
        T.setIdentity();
        
        btVector3 local_inertia(0.0f, 0.0f, 0.0f);
        btDefaultMotionState* my_motion_state = new btDefaultMotionState(T);
        btRigidBody::btRigidBodyConstructionInfo rb_info(0.0f, my_motion_state, collision_shape, local_inertia);
        rb_info.m_additionalDamping = true;
        btRigidBody* convex_hull_rb = new btRigidBody(rb_info);
        
        m_dynamicsWorld->addRigidBody(convex_hull_rb);
    }
    {
        // 4
        btVector3 hullPoints[4] = {btVector3(10.0,0.0,-5.0),btVector3(-10.0,0.0,-5.0)};
        btConvexHullShape hullB;
        hullB.addPoint(hullPoints[0]);
        hullB.addPoint(hullPoints[1]);
        
        btCollisionShape* collision_shape = new btConvexHullShape(hullB);
        
        btTransform T;
        T.setIdentity();
        
        btVector3 local_inertia(0.0f, 0.0f, 0.0f);
        btDefaultMotionState* my_motion_state = new btDefaultMotionState(T);
        btRigidBody::btRigidBodyConstructionInfo rb_info(0.0f, my_motion_state, collision_shape, local_inertia);
        rb_info.m_additionalDamping = true;
        btRigidBody* convex_hull_rb = new btRigidBody(rb_info);
        
        m_dynamicsWorld->addRigidBody(convex_hull_rb);
    }
}
void	RollingFrictionDemo::clientResetScene()
{
	exitPhysics();
	initPhysics();
}
	

void	RollingFrictionDemo::exitPhysics()
{

	//cleanup in the reverse order of creation/initialization

	//remove the rigidbodies from the dynamics world and delete them
	int i;
	for (i=m_dynamicsWorld->getNumCollisionObjects()-1; i>=0 ;i--)
	{
		btCollisionObject* obj = m_dynamicsWorld->getCollisionObjectArray()[i];
		btRigidBody* body = btRigidBody::upcast(obj);
		if (body && body->getMotionState())
		{
			delete body->getMotionState();
		}
		m_dynamicsWorld->removeCollisionObject( obj );
		delete obj;
	}
	
}




